# SDRAM Controller Design Document

## Rationale

This SDRAM Conotroller is a low performance, low resource SDRAM Controller.

It should support only the basic features neccesary for it to be usable in an
FPGA or ASIC system.

## Required Features

* AXI Lite Interface
    - AXI Lite is used instead of AXI Full for simplicity. There are no bursts
      and no reordering of accesses
    - AXI Lite has a greatly reduced number of signals
* Support different external and internal bitwidths
    - Memory chips may not be the same data width as the desired internal width.
      For example, a common memory chip may have an 8 or 16 bit bus, while the
      internal AXI Lite data width would be 32 bits.
    - The memory controller should be able to take a 32 bit read/write and
      convert that into 1/2/4 reads/writes to the memory chip, taking into
      account write strobes and data masks.
    - In this case, we can support sdram bursts.
* Operate at 200MHz PHY speed, variable AXI speed
    - The phy will always operate at 200MHz, but the AXI speed can run slower.
    - Incoming AXI Requests go to an async FIFO which handles the clock domain
      crossing into the 200MHz DRAM domain.


## Architecture

On the AXI Side, there will be fifos for the incoming and outgoing transfers.
This increases latency, but can more fully utilize the sdram.

The incoming AXI Accesses go to the command generator, which will generate the
requisite read/write commands. These are sent to the PHY section, which generates
the signals for the chip as well as maintains the correct timings. The Refresh
Generator contains a timer which generates auto refresh requests at a specified
interval, which depends on the chip.

![sdram_architecture](sdram_architecture.drawio.png)

### Command Generator

These are the SDRAM Commands
* Bank Activate
* Bank Precharge (Deactivate)
* Precharge All (Deactivate All)
* Read
* Read + Auto Precharge
* Write
* Write + Auto Precharge
* Mode Register Set
* NOP
* Burst Stop
* Device Deselect
* Auto Refresh

We do not support low power or self refresh modes.

For simplicty, we will only use the read/write + precharge commands. Leaving banks
activated can increase performance if you access that row again, but that requires
us to remember which banks are open.

For the mode register, this is dependent on the specific chip being used, but we
will certainly need to set the CAS latency and the burst length. Both of these
will be set as parameters and be baked into the startup sequence.

The Refresh Generator supplies two signals to the Command Generator, refresh request
and init reqeust. Refresh request simply causes an auto refresh command to be issued.

An init reqeust will cause a precharge all, then mode register set command to
be issued. After reset, the block will wait idle until this init request is generated.
Before then, no axi requests will be accepted.

### AXI Interface

The AXI Interface acts as an AXI Lite Slave and convertes the AXI
Interface into a single command which gets written into the command queue.

For reads, this writes the address. For writes, it waits until it has seen
both the address and the data, then writes both to the queue. This is a
low performance controller, so you cannot write multiple addresses and then
multiple data. Once the write address is seen, awready goes low until wvalid
is seen.

### Request FIFO

The request FIFO queues requests until the command generator is ready to
act on them. For reads, a single request is added to the fifo, which contains
a bit for "read", and the address. 

WIDTH= 1+DATA_WIDTH+ADDR_WIDTH-clog2(DATA_WIDTH/8).

The upper bit is 0 for read and 1 for write. The lower bits are address, and
the middle bits are the data. It is done this way since reads only use
use the address, so it gets the better bit position.
which is 32. In most cases, the data width will be larger than this
address width, especially if the data width is 32 since they don't really
make SDRAMS that big without going to DDR, which we do not support.

### Response FIFO

The response fifo only stores data and response, so its width is 
1+2+DATA_WIDTH. The extra bits are 1 or read or write, and 2 for the response.
Even though writes don't respond with any data, we still have to return a
write response. If, for example, an invalid address makes it to the controller,
the invalid response will pass through the request fifo, the command generator,
the phy, and all the way back to the response FIFO. In other words, don't do that.1

### Refresh Generator

The Refresh Generator is responsible for auto-refreshing the SDRAM, as well as
handling the startup sequence. It maintains a timer which at its interval,
issues a refresh request to the command generator. The command generator 
will then issue an auto-refresh command. Because we only use the auto-precharge
commands, we do not need to issue the precharge-all command.

The Refresh Generator also handles the startup sequence timer. This simply waits
for the specified amount of time(200us, for example), before sending an init
to the command generator.

### PHY

The PHY takes in the commands generated by the command generator and turns them
into the required signals for the SDRAM, as well as maintain the timings. This
means that the command generator can generate, e.g. a read command followed
immediately by another read command, without any NOPs in between. The PHY will
issue the first read, issue the correct number of NOPs (burst length), then
issue the second read. At the same time, it will either transmit or receive
data on the data bus.

The AXI Address is turned into the sdram address in the PHY. It comes from
splitting up the address into {bank, row, col}.

After the command is complete, the phy will output a command done signal, and
the received data, if any. This can then go to the AXI FIFO as either the read
data/response, or a write response.