import cocotb

from cocotb.clock import Clock
from cocotb.triggers import Timer, RisingEdge, FallingEdge

from cocotbext.axi import AxiLiteMaster, AxiLiteBus

import cocotb.utils

import logging


CLK_PERIOD_200 = 5

class TB:
    def __init__(self, dut):
        self.dut = dut

        self.log = logging.getLogger("cocotb.tb")
        self.log.setLevel(logging.INFO)

        cocotb.start_soon(Clock(dut.i_clk_sys, CLK_PERIOD_200, units="ns").start())
        self.axil = AxiLiteMaster(AxiLiteBus.from_prefix(dut, ""), dut.i_clk_sys, dut.i_rst_sys)


    async def cycle_reset(self):
        self.dut.i_rst_sys.setimmediatevalue(0)
        await RisingEdge(self.dut.i_clk_sys)
        await RisingEdge(self.dut.i_clk_sys)
        self.dut.i_rst_sys.value = 1
        await RisingEdge(self.dut.i_clk_sys)
        await RisingEdge(self.dut.i_clk_sys)
        self.dut.i_rst_sys.value = 0
        await RisingEdge(self.dut.i_clk_sys)
        await RisingEdge(self.dut.i_clk_sys)


@cocotb.test()
async def test_sanity(dut):
    tb = TB(dut)

    await tb.cycle_reset()

    await Timer(250, "us")

    await tb.axil.write_dword(0xabc0, 0x1234678)