module sdram_refresh_generator #(
    parameter TPOWERUP_NS = 200000,
    parameter TREF_MS = 64,
    parameter REFRESH_COUNT = 8192
)(
    input  logic    i_clk_sys,
    input  logic    i_rst_sys,

    output logic    o_ref_req,
    output logic    o_init_req
);

localparam CLK200_NS = 5;
localparam TREF_NS = TREF_MS * 1000000;

localparam TPOWERUP_CLOCKS = TPOWERUP_NS/CLK200_NS;
localparam TREF_CLOCKS = (TREF_NS/REFRESH_COUNT)/CLK200_NS;

`ifdef SIM
initial begin
    $display("TPOWERUP Clocks: %d", TPOWERUP_CLOCKS);
    $display("TREF Clocks: %d", TREF_CLOCKS);
end
`endif

logic [31:0] pwrup_counter, pwrup_counter_next;
logic [31:0] ref_counter, ref_counter_next;

logic ref_req_next;

always_ff @(posedge i_clk_sys) begin
    if (i_rst_sys) begin
        pwrup_counter <= '0;
        ref_counter <= '0;
        o_ref_req <= '0;
    end else begin
        pwrup_counter <= pwrup_counter_next;
        ref_counter <= ref_counter_next;
        o_ref_req <= ref_req_next;
    end
end

always_comb begin
    if (pwrup_counter < TPOWERUP_CLOCKS) begin
        pwrup_counter_next = pwrup_counter + 1;
        o_init_req = '0;
    end else if (pwrup_counter == TPOWERUP_CLOCKS) begin
        pwrup_counter_next = pwrup_counter + 1;
        o_init_req = '1;
    end else begin
        pwrup_counter_next = pwrup_counter;
        o_init_req = '0;
    end

    if (ref_counter == TREF_CLOCKS-1) begin
        ref_counter_next = '0;
        ref_req_next = '1;
    end else begin
        ref_counter_next = ref_counter + 1;
        ref_req_next = '0;
    end
end

endmodule